use std::time::Instant;

use crate::{math::safe_u64_to_f64, sysfs::Netdev};

pub(crate) struct LinkSpeed {
    netdev: Netdev,
    rx_bytes: u64,
    tx_bytes: u64,
    time: Instant,
}

pub(crate) struct MeasurementSpeed {
    pub(crate) rx: f64,
    pub(crate) tx: f64,
}
struct MeasurementDiff {
    rx: u64,
    tx: u64,
}

impl LinkSpeed {
    pub fn new(name: &str) -> eyre::Result<Self> {
        let netdev = Netdev::from_name(name)?;

        Ok(Self {
            netdev,
            rx_bytes: 0,
            tx_bytes: 0,
            time: Instant::now(),
        })
    }

    fn update(&mut self) -> eyre::Result<MeasurementDiff> {
        let rx_bytes = self.netdev.rx_bytes()?;
        let tx_bytes = self.netdev.tx_bytes()?;

        let mut rx_diff = rx_bytes - self.rx_bytes;
        let mut tx_diff = tx_bytes - self.tx_bytes;

        if self.rx_bytes == 0 {
            self.rx_bytes = rx_bytes;
            rx_diff = 0;
        }

        if self.tx_bytes == 0 {
            self.tx_bytes = tx_bytes;
            tx_diff = 0;
        }

        self.rx_bytes = rx_bytes;
        self.tx_bytes = tx_bytes;

        Ok(MeasurementDiff {
            rx: rx_diff,
            tx: tx_diff,
        })
    }

    pub fn get_measurement(&mut self) -> eyre::Result<MeasurementSpeed> {
        let elapsed = self.time.elapsed().as_secs_f64();
        let diff = self.update()?;
        let rx_speed = safe_u64_to_f64(diff.rx)? / elapsed;
        let tx_speed = safe_u64_to_f64(diff.tx)? / elapsed;
        self.time = Instant::now();
        Ok(MeasurementSpeed {
            rx: rx_speed,
            tx: tx_speed,
        })
    }
}

impl Iterator for LinkSpeed {
    type Item = MeasurementSpeed;

    fn next(&mut self) -> Option<Self::Item> {
        let measurement = self.get_measurement();
        match measurement {
            Ok(measurement) => Some(measurement),
            Err(e) => panic!("Error getting measurement: {e}"),
        }
    }
}
